# $Id: movie.py,v 1.4 2003/09/30 23:43:47 travo Exp $
# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from gist import *
from shapetest import *
from yorick import *

#
#  MOVIE.PY
#  Support functions for making animated sequences.
#
#  $Id: movie.py,v 1.4 2003/09/30 23:43:47 travo Exp $
#

#    Copyright (c) 1997.  The Regents of the University of California.
#                  All rights reserved.

def movie (draw_frame, time_limit = 120., min_interframe = 0.0,
   bracket_time = array ([2., 2.], Float ), lims = None, timing = 0) :

   """
   movie (draw_frame, time_limit = 120., min_interframe = 0.0,
      bracket_time = array ([2., 2.], Float ), lims = None, timing = 0)
     runs a movie based on the given DRAW_FRAME function.  The movie
     stops after a total elapsed time of TIME_LIMIT seconds, which
     defaults to 60 (one minute), or when the DRAW_FRAME function
     returns zero.

     note: All but the first argument are keyword arguments, with
     defaults as shown.

     def draw_frame(1) :
       # Input argument i is the frame number.
       # draw_frame should return non-zero if there are more
       # frames in this movie.  A zero return will stop the
       # movie.
       # draw_frame must NOT include any fma command if the
       # making_movie variable is set (movie sets this variable
       # before calling draw_frame)

     If MIN_INTERFRAME is specified, a pause will be added as
     necessary to slow down the movie.  MIN_INTERFRAME is a time
     necessary to slow down the movie.  MIN_INTERFRAME is a time
     in seconds (default 0).

     The keyword bracket_time= (again a time in seconds) can be
     used to adjust the duration of the pauses after the first
     and last frames.  It may also be a two element array [beg, end].
     If the pause at the end is greater than five seconds, you will
     be prompted to explain that hitting <RETURN> will abort the final
     pause. Well, the Python version does not have this capability.

     timing = 1 enables a timing printout for your movie.

     If every frame of your movie has the same limits, use the
     limits command to fix the limits before you call movie.

   BUG:  If you hit <RETURN> to start a movie early, it will not
         pause at the end of the movie at all.  You probably should
         not use long initial pauses.
   """

   global movie_timing, making_movie
   if is_scalar (bracket_time) :
      bracket_time = array ( [bracket_time, bracket_time], Float )
   elif len (bracket_time) == 1 :
      bracket_time = array ( [bracket_time [0], bracket_time [0]], Float )

   elapsed = zeros (3)
   this_frame = zeros (3)

   window (wait = 1, style = "nobox.gs") # make sure window is ready to draw

   fma ( )
   animate (1)
   making_movie = 1

   i = 0
   if time == 1 :
      elapsed = timer_ (elapsed)
      elapsed0 = array(elapsed, copy = 1)
   i = i + 1
   more = draw_frame (i)
   if lims != None:
      limits (lims [0], lims [1], lims [2], lims [3])
   else:
      limits (square = 1)
   fma ( )
   if time == 1 :
      [elapsed, this_frame] = timer_ (elapsed, this_frame)
      wait = bracket_time [0] - this_frame [2]
      waited = waited0 = 0.0
      if wait > 0 :
         if wait > 5 :
            print "Movie starts in", wait, "secs, or when you hit <RETURN>"
         time.sleep (wait)
         waited0 = waited0 + wait
      else :
         wait = min_interframe - this_frame [2]
         if wait > 0 :
            time.sleep (wait)
            waited0 = waited0 + wait

   while (more) :
      this_frame = zeros (3)
      i = i + 1
      more = draw_frame (i)
      if lims != None:
         limits (lims [0], lims [1], lims [2], lims [3])
      else :
         limits (square = 1)
      fma ( )
      if not more :
         break
      if timing == 1 :
         [elapsed, this_frame] = timer_ (elapsed, this_frame)
         if (not more or (elapsed [2] - elapsed0 [2] > time_limit)) :
            break
         wait = min_interframe - this_frame [2]
         if wait > 0 :
            time.sleep (wait)
            waited0 = waited0 + wait

   if timing == 1 :
      wait = bracket_time [1] - this_frame [2]
      if wait > 0 :
         if wait > 5 :
            print "Holding last frame for", wait, "secs, or hit <RETURN>"
         time.sleep (5)      # wait)
         waited0 = waited0 + wait

      elapsed = timer_ (elapsed)

      e = elapsed - elapsed0
      movie_timing = [e [0], e [1], e [2], i, waited, waited0]

      movie_stats ()

   animate (0)

def movie_stats ( *timing ) :

   """
   movie_stats ( ) or movie_stats ( timing )
     prints statistics from the last movie command, or from the
     command which produced TIMING.  TIMING is the contents of the
     movie_timing external variable after the movie command completes.
   """

   if len (timing) > 0 :
      timing = timing [0]
   else :
      timing = movie_timing
   cpu = timing [0] + timing [1]
   print "user", timing [0], "system", timing [1]
   wall = timing [2]
   nframes = timing [3]
   waited = timing [4] + timing [5]
   wait = timing [5]

   print "  Wall(sec)  Wait(sec)  CPU(sec)"
   print "  %9.3f  %9.3f  %8.3f   %ld frames\n" % (wall, waited, cpu, nframes)
   print "  %9.3f  %9.3f  %8.3f   per frame\n" % \
      ( (wall - waited) / nframes, wait / ( (nframes > 1) * (nframes - 1) + \
      (nframes <= 1) * 1), cpu / nframes)



#!/usr/bin/env python

import os
from scipy_distutils.misc_util import get_path, default_config_dict

def configuration(parent_package='',parent_path=None):
    package = 'plt'
    local_path = get_path(__name__,parent_path)
    config = default_config_dict(package,parent_package)

    data = ['lena.dat']
    data = [os.path.join(local_path,x) for x in data]               
    config['data_files'].extend( [(os.path.join(parent_package,'plt'),data)])
  
    return config

if __name__ == '__main__':
    from scipy_distutils.core import setup
    setup(**configuration())



import unittest
import sys
from scipy_test.testing import *
set_package_path()
import scipy_base;reload(scipy_base)
from scipy_base import fastumath;reload(fastumath)
del sys.path[0]


class test_maximum(ScipyTestCase):
    def check_reduce_complex(self):
        x = [1,2]
        assert_equal(fastumath.maximum.reduce([1,2j]),1)

class test_minimum(ScipyTestCase):
    def check_reduce_complex(self):
        x = [1,2]
        assert_equal(fastumath.minimum.reduce([1,2j]),2j)

if __name__ == "__main__":
    ScipyTest('scipy_base.fastumath').run()



#!/usr/bin/env python

from __future__ import nested_scopes
import os
import sys
import re
from distutils.dep_util import newer_group, newer
from glob import glob
from os.path import join

#-------------------
# To skip wrapping single precision atlas/lapack/blas routines, set
# the following flag to True:
skip_single_routines = 0

# Some OS distributions (e.g. Redhat, Suse) provide a blas library that
# is built using incomplete blas sources that come with lapack tar-ball.
# In order to use such a library in scipy.linalg, the following flag
# must be set to True:
using_lapack_blas = 0

#--------------------

def configuration(parent_package='',parent_path=None):
    from scipy_distutils.core import Extension
    from scipy_distutils.misc_util import dot_join, get_path, default_config_dict
    from scipy_distutils.system_info import get_info, dict_append, NotFoundError

    from interface_gen import generate_interface

    package = 'linalg'
    config = default_config_dict(package,parent_package)
    local_path = get_path(__name__,parent_path)
    def local_join(*paths):
        return os.path.join(*((local_path,)+paths))

    lapack_opt = get_info('lapack_opt')

    if not lapack_opt:
        raise NotFoundError,'no lapack/blas resources found'

    atlas_version = ([v[3:-3] for k,v in lapack_opt.get('define_macros',[]) \
                      if k=='ATLAS_INFO']+[None])[0] 
    if atlas_version:
        print 'ATLAS version',atlas_version

    target_dir = ''
    skip_names = {'clapack':[],'flapack':[],'cblas':[],'fblas':[]}
    if skip_single_routines:
        target_dir = 'dbl'
        skip_names['clapack'].extend(\
            'sgesv cgesv sgetrf cgetrf sgetrs cgetrs sgetri cgetri'\
            ' sposv cposv spotrf cpotrf spotrs cpotrs spotri cpotri'\
            ' slauum clauum strtri ctrtri'.split())
        skip_names['flapack'].extend(skip_names['clapack'])
        skip_names['flapack'].extend(\
            'sgesdd cgesdd sgelss cgelss sgeqrf cgeqrf sgeev cgeev'\
            ' sgegv cgegv ssyev cheev slaswp claswp sgees cgees'
            ' sggev cggev'.split())
        skip_names['cblas'].extend('saxpy caxpy'.split())
        skip_names['fblas'].extend(skip_names['cblas'])
        skip_names['fblas'].extend(\
            'srotg crotg srotmg srot csrot srotm sswap cswap sscal cscal'\
            ' csscal scopy ccopy sdot cdotu cdotc snrm2 scnrm2 sasum scasum'\
            ' isamax icamax sgemv cgemv chemv ssymv strmv ctrmv'\
            ' sgemm cgemm'.split())

    if using_lapack_blas:
        target_dir = join(target_dir,'blas')
        skip_names['fblas'].extend(\
            'drotmg srotmg drotm srotm'.split())

    if atlas_version=='3.2.1_pre3.3.6':
        target_dir = join(target_dir,'atlas321')
        skip_names['clapack'].extend(\
            'sgetri dgetri cgetri zgetri spotri dpotri cpotri zpotri'\
            ' slauum dlauum clauum zlauum strtri dtrtri ctrtri ztrtri'.split())
    elif atlas_version>'3.4.0' and atlas_version<='3.5.12':
        skip_names['clapack'].extend('cpotrf zpotrf'.split())

    def generate_pyf(extension, build_dir):
        name = extension.name.split('.')[-1]
        target = join(build_dir,target_dir,name+'.pyf')
        if name[0]=='c' and atlas_version is None and newer(__file__,target):
            f = open(target,'w')
            f.write('python module '+name+'\n')
            f.write('usercode void empty_module(void) {}\n')
            f.write('interface\n')
            f.write('subroutine empty_module()\n')
            f.write('intent(c) empty_module\n')
            f.write('end subroutine empty_module\n')
            f.write('end interface\nend python module'+name+'\n')
            f.close()
            return target
        if newer_group(extension.depends,target):
            generate_interface(name,
                               extension.depends[0],
                               target,
                               skip_names[name])
        return target

    # fblas:
    ext_args = {'name': dot_join(parent_package,package,'fblas'),
                'sources': [generate_pyf,
                            local_join('src','fblaswrap.f')],
                'depends': map(local_join,['generic_fblas.pyf',
                                           'generic_fblas1.pyf',
                                           'generic_fblas2.pyf',
                                           'generic_fblas3.pyf',
                                           'interface_gen.py'])
                }
    dict_append(ext_args,**lapack_opt)
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)

    # cblas:
    ext_args = {'name': dot_join(parent_package,package,'cblas'),
                'sources': [generate_pyf],
                'depends': map(local_join,['generic_cblas.pyf',
                                           'generic_cblas1.pyf',
                                           'interface_gen.py'])
                }
    dict_append(ext_args,**lapack_opt)
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)

    # flapack:
    ext_args = {'name': dot_join(parent_package,package,'flapack'),
                'sources': [generate_pyf],
                'depends': map(local_join,['generic_flapack.pyf',
                                           'flapack_user_routines.pyf',
                                           'interface_gen.py'])
                }
    dict_append(ext_args,**lapack_opt)
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)

    # clapack:
    ext_args = {'name': dot_join(parent_package,package,'clapack'),
                'sources': [generate_pyf],
                'depends': map(local_join,['generic_clapack.pyf',
                                           'interface_gen.py'])
                }
    dict_append(ext_args,**lapack_opt)
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)

    # _flinalg:
    ext_args = {'name':dot_join(parent_package,package,'_flinalg'),
                'sources':[local_join('src','det.f'),
                           local_join('src','lu.f')]
                }
    dict_append(ext_args,**lapack_opt)
    config['ext_modules'].append(Extension(**ext_args))

    # calc_lwork:
    ext_args = {'name':dot_join(parent_package,package,'calc_lwork'),
                'sources':[local_join('src','calc_lwork.f')],
                }
    dict_append(ext_args,**lapack_opt)
    config['ext_modules'].append(Extension(**ext_args))

    # atlas_version:
    ext_args = {'name':dot_join(parent_package,package,'atlas_version'),
                'sources':[os.path.join(local_path,'atlas_version.c')]}
    dict_append(ext_args,**lapack_opt)
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)

    # iterative methods
    methods = ['BiCGREVCOM.f.src',
               'BiCGSTABREVCOM.f.src',
               'CGREVCOM.f.src',
               'CGSREVCOM.f.src',
#               'ChebyREVCOM.f.src',
               'GMRESREVCOM.f.src',
#               'JacobiREVCOM.f.src',
               'QMRREVCOM.f.src',
#               'SORREVCOM.f.src'
               ]
    Util = ['STOPTEST2.f.src','getbreak.f.src']
    sources = Util + methods + ['_iterative.pyf.src']
    ext_args = {
        'name': dot_join(parent_package,package,'_iterative'),
        'sources': [local_join('iterative',x) for x in sources]
        }
    dict_append(ext_args, **lapack_opt)
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)    

    return config

if __name__ == '__main__':
    from scipy_distutils.core import setup
    from linalg_version import linalg_version

    setup(version=linalg_version,
          **configuration(parent_path=''))



# Interface to Constrained Optimization By Linear Approximation

from __future__ import nested_scopes
import _cobyla

def fmin_cobyla(func, x0, cons, args=(), consargs=None, rhobeg=1.0, rhoend=1e-4,
                iprint=1, maxfun=1000):
    """
    Minimize a function using the Contrained Optimization BY Linear Approximation
    (COBYLA) method

    Arguments:

    func     -- function to minimize. Called as func(x, *args)

    x0       -- initial guess to minimum

    cons     -- a list of functions that all must be >=0 (a single function
                if only 1 constraint)

    args     -- extra arguments to pass to function

    consargs -- extra arguments to pass to constraints (default of None means
                use same extra arguments as those passed to func).  Use () for no
                extra arguments.

    rhobeg --  reasonable initial changes to the variables

    rhoend --  final accuracy in the optimization (not precisely guaranteed)

    iprint  -- controls the frequency of output: 0 (no output),1,2,3

    maxfun  -- maximum number of function evaluations.


    Returns:

    x -- the minimum
    
    """
    err = "cons must be a list of callable functions or a single"\
              " callable function."
    n = len(x0)
    if isinstance(cons, list):
        m = len(cons)
        for thisfunc in cons:
            if not callable(thisfunc):
                raise TypeError, err
    elif callable(cons):
        m = 1
        cons = [cons]
    else:
        raise TypeError, "cons must be a list of callable functions or a single"\
              " callable function."

    if consargs is None:
        consargs = args
        
    def calcfc(x, con):
        f = func(x, *args)
        k = 0
        for constraints in cons:
            con[k] = constraints(x,*consargs)
            k += 1
        return f

    xopt = _cobyla.minimize(calcfc, m=m, x=x0,rhobeg=rhobeg,rhoend=rhoend,iprint=iprint,
                            maxfun=maxfun)
    
    return xopt
